﻿#region Using directives

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using System.Threading;
using WindowsApplication1.localhost;

#endregion

namespace WindowsApplication1
{
    partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            MasterInfos infos = getInfos();

            initCmbCustomer(infos.customers);
            initCmbEmployee(infos.employees);
            initProductCol(infos.products);

        }

        private void initCmbCustomer(Customer[] custs)
        {
            cmbCustomer.DisplayMember = "name";
            cmbCustomer.ValueMember = "id";

            cmbCustomer.DataSource = CustomerRec.getCustomerRecs(custs);
        }

        private void initCmbEmployee(Employee[] employees)
        {
            cmbEmployee.DisplayMember = "name";
            cmbEmployee.ValueMember = "id";

            cmbEmployee.DataSource = EmployeeRec.getEmployeeRecs(employees);
        }

        private void initProductCol(Product[] products)
        {
            DataGridViewComboBoxColumn cmbCol = (DataGridViewComboBoxColumn)dataGridView1.Columns[0];
            cmbCol.DisplayMember = "name";
            cmbCol.ValueMember = "id";
            cmbCol.DataSource = ProductRec.getProductRecs(products);
        }

        private SalesServiceService getService() {
            return new SalesServiceService();
        }

        private MasterInfos getInfos()
        {
            return getService().getMasterInfos();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            SalesServiceService service = getService();
            Sales sales = getSales();

            //Thread.Sleep(200);

            service.insertSales(sales);
//            getService().insertSales(getSales());
        }

        private Sales getSales()
        {
            Sales sales = new Sales();
            sales.header = getHeader();
            sales.details = getDatails();
            return sales;
        }

        private SalesHeader getHeader()
        {
            SalesHeader header = new SalesHeader();
            header.customerId = Convert.ToInt32(cmbCustomer.SelectedValue);
            header.employeeId = Convert.ToInt32(cmbEmployee.SelectedValue);
            header.salesDate = dtString(dateTimePicker1.Value);
            return header;
        }

        private SalesDetail[] getDatails()
        {
            ArrayList list = new ArrayList();
            for (int i = 0; i < dataGridView1.RowCount; i++)
            {
                DataGridViewRow dgvRow = dataGridView1.Rows[i];
                if (Convert.ToInt32(dgvRow.Cells[0].Value) > 0)
                {
                    SalesDetail detail = new SalesDetail();
                    detail.productId = Convert.ToInt32(dgvRow.Cells[0].Value);
                    detail.quantity = Convert.ToInt32(dgvRow.Cells[1].Value);
                    list.Add(detail);
                }
            }

            return (SalesDetail[])list.ToArray(typeof(SalesDetail));
        }

        private string dtString(DateTime dt)
        {
            return dt.Year.ToString() + "-" + zeropadStr(dt.Month) + "-" + zeropadStr(dt.Day);
        }

        private string zeropadStr(int val) {
            string tmp = "0" + val.ToString();
            return tmp.Substring(tmp.Length - 2, 2);
        }
    }

    public class CustomerRec
    {
        private Customer customer;

        public static ArrayList getCustomerRecs(Customer[] customers)
        {
            ArrayList list = new ArrayList();

            for (int i = 0; i < customers.Length; i++) {
                list.Add(new CustomerRec(customers[i]));
                }

            return list;
        }

        public CustomerRec(Customer customer) {
            this.customer = customer;
        }

        public int id {
            get {return customer.id;}
        }

        public string name {
            get {return customer.name;}
        }
    }

    public class EmployeeRec
    {
        private Employee employee;

        public static ArrayList getEmployeeRecs(Employee[] employees)
        {
            ArrayList list = new ArrayList();

            for (int i = 0; i < employees.Length; i++) {
                list.Add( new EmployeeRec(employees[i]));
                }
            return list;
        }

        public EmployeeRec(Employee employee)
        {
            this.employee = employee;
        }

        public int id
        {
            get { return employee.id; }
        }

        public string name
        {
            get { return employee.name; }
        }
    }

    public class ProductRec
    {
        private Product product;

        public static ArrayList getProductRecs(Product[] products) {
            ArrayList list = new ArrayList();

            for (int i = 0; i < products.Length; i++) {
                list.Add(new ProductRec(products[i]));
                }
            return list;
        }

        public ProductRec(Product product)
        {
            this.product = product;
        }

        public int id
        {
            get
            {
                return product.id;
            }
        }

        public string name
        {
            get { return product.name; }
        }
    }

}